MetaSpaKmeans <-
function(x, K=NULL, wbounds=NULL, nstart=20, ntrial=1, maxiter=20,  lambda=1/2, method="exhaustive",sampleSizeAdjust = FALSE,wsPre=NULL, silence=FALSE){
      # x is list of data, (nxp) for each study
      # wbounds is a vector of L1 constraints on w, of the form  sum(abs(w))<=wbounds[i]    
	  # nstart: initial Kmeans searching space.
	  # maxiter: maximum number of iterations.
	  # lambda: a tuning parameter keeping the balance between separation and matching
	  # method: contains exhaustive, linear exhaustive and linear exhaustive + simulated anealling.
	  # wsPre: if specified, the initial weight.
	  # silence: if print some details


	  ## check input
	  
      ## get some basic information
	  numStudies = length(x)

	  ## create result
	  bestOut <- vector('list',length(wbounds))
      out <- vector('list',length(wbounds))
      Cs0 <- list()
      tss.x <- list()

	  for(i in 1:numStudies){
		  ## total sum of square for each study
		  tss.x[[i]] <- apply(scale(x[[i]], center=TRUE, scale=FALSE)^2, 2, sum)
	  	}

      for(atrail in 1:ntrial){
		  #
	      ## initialize    
	      ## initialize cluster by KMeans
	      ## initialize w
	      if(is.null(wsPre)){
	        for (i in 1:numStudies){
	            Cs0[[i]] <- kmeans(x[[i]], centers=K, nstart=nstart)$cluster
	        }
	      } else {
	        if(length(wsPre)!=ncol(x[[1]]))
	          stop("length of wsPre differs from number of genes")
	        if(is.null(names(wsPre)))
	          stop("there is no name for wsPre")
	        if(any(names(wsPre)!=colnames(x[[1]])))
	          stop("name of wsPre differs from gene name")
	        for (i in 1:numStudies)
	  	    Cs0[[i]] <- weightedKMeans(x=t(x[[i]]), K=K, ws=wsPre, tss.x=tss.x[[i]])
	      }	  
		  #
		  for(w in 1:length(wbounds)){
			  awbound = wbounds[w]
			  if(!is.null(wsPre)){
				  ws=wsPre
				} else {
		            ws <- rep(1, ncol(x[[1]]))/(ncol(x[[1]]))*awbound # Start with equal weights on each feature    			  	
				}

			  ws.old <- rnorm(ncol(x[[1]]))
			  store.ratio <- NULL
			  niter <- 0
			  trace <- list()
			  Cs = Cs0
		  
	  	    ## iterate until converge or maxiteration
	  	    while((sum(abs(ws-ws.old))/sum(abs(ws.old)))>1e-4 && niter<maxiter){
	  		  cat("iteration:")
	  		  cat(niter)
	  		  cat("\n")
	  		  cat("convergence criteria: ")
	  		  cat(sum(abs(ws-ws.old))/sum(abs(ws.old)))
	  		  cat("\n")
	  
	  	      niter <- niter+1
	  	      ws.old <- ws  
	  	      if(niter>1) Cs <- UpdateCs(x, K, ws, Cs,nstart=nstart, tss.x) # if niter=1, no need to update!!
      
	  	      fmatch = patternMatch(x,Cs,ws,method=method,silence=silence)
	  	      ratio = GetRatio(x, Cs, tss.x,sampleSizeAdjust=sampleSizeAdjust)
	  	      ws <- UpdateWs(x, Cs, awbound, ratio, lambda*(fmatch$perEng+1)/2)
	  	      store.ratio <- c(store.ratio, sum(ratio*ws))
		  		  
	  	    }
		    score = sum((ratio + lambda*(fmatch$perEng+1)/2)*ws)
		    names(ws) <- colnames(x[[1]])
		    out[[w]] <- list(ws=ws, Cs=fmatch$matchCs, wbound=awbound, score=score) 		
			if(is.null(bestOut[[w]])){
				bestOut[[w]] = out[[w]]
			} else {
				if(bestOut[[w]]$score < out[[w]]$score )	
					bestOut[[w]] = out[[w]]
			}
		}    
			
	}
	if(length(bestOut)==1){
		bestOut=bestOut[[1]]
	} 
    class(bestOut) <- "metaSparseKmeans"
    return(bestOut)    
  }
